<%= form_for application, url: doorkeeper_submit_path(application), html: {class: 'form-horizontal', role: 'form'} do |f| %>
  <% if application.errors.any? %>
    <div class="alert alert-danger" data-alert><p><%= t('doorkeeper.applications.form.error') %></p></div>
  <% end %>

  <%= content_tag :div, class: "form-group#{' has-error' if application.errors[:name].present?}" do %>
    <%= f.label :name, class: 'col-sm-2 control-label' %>
    <div class="col-sm-10">
      <%= f.text_field :name, class: 'form-control' %>
      <%= doorkeeper_errors_for application, :name %>
    </div>
  <% end %>

  <%= content_tag :div, class: "form-group#{' has-error' if application.errors[:redirect_uri].present?}" do %>
    <%= f.label :redirect_uri, class: 'col-sm-2 control-label' %>
    <div class="col-sm-10">
      <%= f.text_area :redirect_uri, class: 'form-control' %>
      <%= doorkeeper_errors_for application, :redirect_uri %>
      <span class="help-block">
         <%= t('doorkeeper.applications.help.redirect_uri') %>
        </span>
      <% if Doorkeeper.configuration.native_redirect_uri %>
          <span class="help-block">
            <%= raw t('doorkeeper.applications.help.native_redirect_uri', native_redirect_uri: "<code>#{ Doorkeeper.configuration.native_redirect_uri }</code>") %>
          </span>
      <% end %>
    </div>
  <% end %>

  <div class="form-group">
    <div class="col-sm-offset-2 col-sm-10">
      <%= f.submit t('doorkeeper.applications.buttons.submit'), class: "btn btn-primary" %>
      <%= link_to t('doorkeeper.applications.buttons.cancel'), oauth_applications_path, :class => "btn btn-default" %>
    </div>
  </div>
<% end %>
